﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Duality;
using Duality.Components.Physics;

namespace Space_Station_Escape
{
    [Serializable]
    public class PoisonGas : Component, ICmpInitializable, ICmpCollisionListener, ICmpUpdatable
    {
        Player currentlyDamagingPlayer;
        float castedAt, lastDamagedAt;

        public void OnInit(Component.InitContext context)
        {
            if (context.Equals(InitContext.Activate))
            {
                this.castedAt = Time.GameTimer.Seconds;
                foreach (ShapeInfo shape in this.GameObj.RigidBody.Shapes)
                {
                    shape.IsSensor = true;
                }
            }
        }

        public void OnShutdown(Component.ShutdownContext context)
        {
        }

        public void OnCollisionBegin(Component sender, CollisionEventArgs args)
        {
            if (currentlyDamagingPlayer == null)
            {
                currentlyDamagingPlayer = sender.GameObj.GetComponent<Player>();
            }
        }

        public void OnCollisionEnd(Component sender, CollisionEventArgs args)
        {
            currentlyDamagingPlayer = null;
        }

        public void OnCollisionSolve(Component sender, CollisionEventArgs args)
        {
        }

        public void OnUpdate()
        {
            if (currentlyDamagingPlayer != null && Time.GameTimer.Seconds - lastDamagedAt >= 1f)
            {
                currentlyDamagingPlayer.heal(-10f);
                lastDamagedAt = Time.GameTimer.Seconds;
            }

            if (Time.GameTimer.Seconds - castedAt >= 100f)
            {
                this.GameObj.DisposeLater();
            }
        }
    }
}
