﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Duality;
using Duality.Components.Physics;
using OpenTK.Input;
using OpenTK;
using System.Collections;
using Duality.Resources;
using Duality.Components;

namespace Space_Station_Escape
{
    [Serializable]
    public class RougeAI : Component, ICmpUpdatable, ICmpInitializable
    {
        /// <summary>
        /// Move left key
        /// </summary>
        public Key MoveLeftKey { get; set; }

        /// <summary>
        /// Move right key
        /// </summary>
        public Key MoveRightKey { get; set; }

        /// <summary>
        /// Jump key
        /// </summary>
        public Key MoveUpKey { get; set; }

        /// <summary>
        /// Move left key
        /// </summary>
        public Key MoveDownKey { get; set; }

        /// <summary>
        /// Move right key
        /// </summary>
        public Key DropBlockKey { get; set; }

        /// <summary>
        /// Make gas.
        /// </summary>
        public Key MakeGasKey { get; set; }

        /// <summary>
        /// Turns lights off
        /// </summary>
        public Key LightsOffKey { get; set; }

        /// <summary>
        /// Speed this AI moves.
        /// </summary>
        public float speed { get; set; }

        float globalLastCast;
        public Dictionary<Key, Tuple<float, float>> keyLastcastCooldownPair;
        bool lightsOff = false;
        float lightsOffLastCast;

        public void OnUpdate()
        {
            RigidBody body = this.GameObj.RigidBody;

            if (DualityApp.Keyboard[this.MoveLeftKey])
            {
                body.LinearVelocity = new Vector2(-7f, body.LinearVelocity.Y);
            }
            else if (DualityApp.Keyboard[this.MoveRightKey])
            {
                body.LinearVelocity = new Vector2(7f, body.LinearVelocity.Y);
            }
            else if (DualityApp.Keyboard[this.MoveDownKey])
            {
                body.LinearVelocity = new Vector2(body.LinearVelocity.X,  7f);
            }
            else if (DualityApp.Keyboard[this.MoveUpKey])
            {
                body.LinearVelocity = new Vector2(body.LinearVelocity.X, -7f);
            }
            else
            {
                body.LinearVelocity = new Vector2(0, 0);
            }

            if (Time.GameTimer.Seconds - globalLastCast >= 1f)
            {
                if (DualityApp.Keyboard[this.DropBlockKey] &&
                    Time.GameTimer.Seconds - keyLastcastCooldownPair[this.DropBlockKey].Item1
                    >= keyLastcastCooldownPair[this.DropBlockKey].Item2)
                {
                    keyLastcastCooldownPair[this.DropBlockKey]
                        = new Tuple<float, float>(
                            Time.GameTimer.Seconds,
                            keyLastcastCooldownPair[this.DropBlockKey].Item2
                            );
                    GameObject newBombBlock = GameRes.Data.BombBlock_Prefab.Res.Instantiate();
                    newBombBlock.Transform.Pos = new Vector3(
                        this.GameObj.Transform.Pos.X,
                        this.GameObj.Transform.Pos.Y,
                        this.GameObj.Transform.Pos.Z
                        );
                    Scene.Current.AddObject(newBombBlock);
                }
                else if (DualityApp.Keyboard[this.MakeGasKey] &&
                    Time.GameTimer.Seconds - keyLastcastCooldownPair[this.MakeGasKey].Item1
                    >= keyLastcastCooldownPair[this.MakeGasKey].Item2)
                {
                    keyLastcastCooldownPair[this.MakeGasKey]
                        = new Tuple<float, float>(
                            Time.GameTimer.Seconds,
                            keyLastcastCooldownPair[this.MakeGasKey].Item2
                            );
                    GameObject newBombBlock = GameRes.Data.PoisonGas_Prefab.Res.Instantiate();
                    newBombBlock.Transform.Pos = new Vector3(
                        this.GameObj.Transform.Pos.X,
                        this.GameObj.Transform.Pos.Y,
                        this.GameObj.Transform.Pos.Z
                        );
                    Scene.Current.AddObject(newBombBlock);
                }
                else if (DualityApp.Keyboard[this.LightsOffKey] &&
                    Time.GameTimer.Seconds - keyLastcastCooldownPair[this.LightsOffKey].Item1
                    >= keyLastcastCooldownPair[this.LightsOffKey].Item2)
                {
                    keyLastcastCooldownPair[this.LightsOffKey]
                        = new Tuple<float, float>(
                            Time.GameTimer.Seconds,
                            keyLastcastCooldownPair[this.LightsOffKey].Item2
                            );
                    foreach (Camera camera in Scene.Current.Cameras)
                    {
                        camera.VisibilityMask = VisibilityFlag.Group2;
                    }

                    lightsOffLastCast = Time.GameTimer.Seconds;
                    lightsOff = true;
                }
            }

            if (lightsOff && Time.GameTimer.Seconds - lightsOffLastCast >= 3f)
            {
                lightsOff = false;
                foreach (Camera camera in Scene.Current.Cameras)
                {
                    camera.VisibilityMask = VisibilityFlag.All;
                }
            }

        }

        public void OnInit(Component.InitContext context)
        {
            if(context.Equals(InitContext.Loaded))
            {
                this.MoveUpKey = Key.W;
                this.MoveDownKey = Key.S;
                this.MoveRightKey = Key.D;
                this.MoveLeftKey = Key.A;
                this.DropBlockKey = Key.E;
                this.MakeGasKey = Key.Q;
                this.LightsOffKey = Key.R;

                this.keyLastcastCooldownPair = new Dictionary<Key, Tuple<float, float>>();
                this.globalLastCast = Time.GameTimer.Seconds;
                keyLastcastCooldownPair[this.DropBlockKey] = new Tuple<float, float>(
                    Time.GameTimer.Seconds,
                    3.0f
                    );
                keyLastcastCooldownPair[this.MakeGasKey] = new Tuple<float, float>(
                    Time.GameTimer.Seconds,
                    5.0f
                    );
                keyLastcastCooldownPair[this.LightsOffKey] = new Tuple<float, float>(
                    Time.GameTimer.Seconds,
                    10.0f
                    );
            }
        }

        public void OnShutdown(Component.ShutdownContext context)
        {
        }


        /// <summary>
        /// Never let it fall below 0
        /// </summary>
        /// <param name="cd"></param>
        /// <returns></returns>
        private float Clamp(float cd)
        {
            if (cd <= 0.0f)
                return 0.0f;
            else return cd;
        }
    }
}
